# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.import datetime

"""Perform validation of output against schemas."""

from lxml import etree
from StringIO import StringIO

from nova import exception
from nova import flags
from nova import log as logging
from nova.api.openstack import common


LOG = logging.getLogger('nova.api.openstack')


FLAGS = flags.FLAGS
flags.DEFINE_bool('validate_xml',
                  True,
                  'Validate XML output?')
flags.DEFINE_bool('validate_xml_for_json',
                  False,
                  'Always generate and validate XML output, even for JSON?')
flags.DEFINE_bool('raise_error_on_validation_warnings',
                  False,
                  'Treat XML validation failures as errors?')
# NOTE(justinsb): Until the OS 1.1 schema is fixed, don't bother checking it
flags.DEFINE_bool('validate_openstack_11',
                  False,
                  'Should we validate openstack 1.1 schema?')


class OutputValidator(object):
    """Perform validation of output."""

    def validate_output(self, content_type, output):
        """Validate the output."""
        if content_type == "application/xml":
            is_valid = self._validate_xml(output)
            if FLAGS.raise_error_on_validation_warnings and not is_valid:
                raise exception.Error(_("XML output was not valid"))
        elif content_type == "application/json":
            pass
        else:
            LOG.warn(_("Unknown content type: %s") % content_type)

    def always_validate_xml(self):
        """Should we always generate and validate XML output?"""
        return FLAGS.validate_xml_for_json

    def _validate_xml(self, xml_text):
        """Validate XML output."""
        doc = etree.parse(StringIO(xml_text))

        xmlns_prefix = doc.getroot().prefix
        xmlns = doc.getroot().nsmap.get(xmlns_prefix, None)

        if xmlns is None:
            LOG.warning(_("No XML namespace for XML: %s") % xml_text)
            return False

        if xmlns == common.XML_NS_V11:
            if not FLAGS.validate_openstack_11:
                return True

        xmlschema = common.load_xml_schema(xmlns)
        if not xmlschema:
            LOG.warning(_("Unknown XML namespace: %s") % xmlns)
            return False

        try:
            xmlschema.assertValid(doc)
        except etree.DocumentInvalid as e:
            LOG.warning(_("XML did not validate: %(e)s\nXML=%(xml_text)s") %
                        locals())
            return False

        return True
